@@ -258,7 +258,8 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         int k;
                         for(k=0; k<vector_size(SFD(nextin).fdbuf.mLines); k++) {
                             if(markfiles[k]) {
-                                if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, k))) {
+                                char* str = vector_item(SFD(nextin).fdbuf.mLines, k);
+                                if(!fd_write(nextout,str, strlen(str))) {
                                     err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                     FREE(markfiles);
@@ -270,7 +271,8 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         }
 
                         if(!flg_mark) {
-                            if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, cursor))) {
+                            char* str = vector_item(SFD(nextin).fdbuf.mLines, cursor);
+                            if(!fd_write(nextout, str, strlen(str))) {
                                 err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 FREE(markfiles);
@@ -279,14 +281,15 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         }
                     }
                     else {
-                        if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, cursor))) {
+                        char* str = vector_item(SFD(nextin).fdbuf.mLines, cursor);
+                        if(!fd_write(nextout, str, strlen(str))) {
                             err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             FREE(markfiles);
                             return FALSE;
                         }
                     }
-                    if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+                    if(SFD(nextin).fdbuf.mBufLen == 0) {
                         runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
                     }
                     else {
@@ -378,14 +381,14 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         /// make control characters visible ///
         char* p = SFD(nextin).fdbuf.mBuf;
-        while(*p) {
+        while(p - SFD(nextin).fdbuf.mBuf < SFD(nextin).fdbuf.mBufLen) {
             if(*p == '\n') {
                 if(!fd_writec(nextin2, *p)) {
                     return FALSE;
                 }
                 p++;
             }
-            else if(*p >= 1 && *p <= 31) {
+            else if(*p >= 0 && *p <= 31) {
                 if(!fd_writec(nextin2, '^')) {
                     return FALSE;
                 }
@@ -522,13 +525,13 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
             else if(key == 10 || key == 13) {
-                if(!fd_write(nextout, SFD(nextin).fdbuf.mBuf)) {
+                if(!fd_write(nextout, SFD(nextin).fdbuf.mBuf, SFD(nextin).fdbuf.mBufLen)) {
                     err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     FREE(markfiles);
                     return FALSE;
                 }
-                if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+                if(SFD(nextin).fdbuf.mBufLen == 0) {
                     runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
                 }
                 else {
@@ -605,7 +608,7 @@ BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     char* buf = readline(prompt);
 
-    if(!fd_write(nextout, buf)) {
+    if(!fd_write(nextout, buf, strlen(buf))) {
         err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
@@ -1192,7 +1195,7 @@ BOOL cmd_fselector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     , strlen(cursor_path) - strlen(cwd_path)+1);
                 xstrncat(cursor_path2, "\n", PATH_MAX+1);
 
-                if(!fd_write(nextout, cursor_path2))
+                if(!fd_write(nextout, cursor_path2, strlen(cursor_path2)))
                 {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     int i;
@@ -1204,7 +1207,7 @@ BOOL cmd_fselector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             else {
                 xstrncat(cursor_path, "\n", PATH_MAX+1);
-                if(!fd_write(nextout, cursor_path))
+                if(!fd_write(nextout, cursor_path, strlen(cursor_path)))
                 {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     int i;
@@ -1231,7 +1234,7 @@ BOOL cmd_fselector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     full_path2[strlen(full_path) - strlen(cwd_path)] = '\n';
                     full_path2[strlen(full_path) - strlen(cwd_path) + 1] = 0;
 
-                    if(!fd_write(nextout, full_path2))
+                    if(!fd_write(nextout, full_path2, strlen(full_path2)))
                     {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         int i;
@@ -1243,7 +1246,7 @@ BOOL cmd_fselector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
                 else {
                     xstrncat(full_path, "\n", PATH_MAX+1);
-                    if(!fd_write(nextout, full_path))
+                    if(!fd_write(nextout, full_path, strlen(full_path)))
                     {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         int i;
